#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 13:51:26 2023

@author: chris
"""
from Orange.data import Table
from Orange.widgets import widget, gui
from Orange.widgets.widget import OWWidget
from Orange.widgets.utils.signals import Input, Output
from orangecontrib.timeseries import Timeseries
import numpy as np

class OWCleanWFDBSignal(OWWidget):
    name = 'Clean WFDB Signal'
    description = 'Fills in 0s with window-statistical values'
    icon = 'icons/clean_wfdb.svg'
    priority = 20
    
    class Inputs:
        time_series = Input("Time series", Table)
        
    class Outputs:
        time_series = Output("Time series", Timeseries)
        
    class Warning(OWWidget.Warning):
        no_aggregations = widget.Msg("No (applicable) aggregations are selected")
        inapplicable_aggregations = \
            widget.Msg("Some aggregations are applicable "
                       "only to sliding window ({})")
        window_to_large = widget.Msg("Window width is too large")
        block_to_large = widget.Msg("Block width is too large")

    class Error(OWWidget.Error):
        migrated_aggregate = widget.Msg(
            "Aggregate was replaced with Moving Transform; "
            "manually re-set the widget")

    want_main_area = False
    
    def __init__(self):
        self.data = None
        # GUI
        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(
            box, "No data on input yet, waiting to get something.")
        self.infob = gui.widgetLabel(box, '')
        
    @Inputs.time_series
    def set_data(self, dataset):
        if dataset is not None:
            #self.data = dataset
            self.infoa.setText("%d instances in input data set" % len(dataset))
            fhr = dataset.get_column('FHR')
            smooth_fhr = OWCleanWFDBSignal.smooth_zeros(fhr)
            xdf, ydf, mdf = list(dataset.to_pandas_dfs())
            xdf["FHR Smooth"] = smooth_fhr.reshape(len(smooth_fhr), 1)
            output = Table.from_pandas_dfs(xdf, ydf, mdf)
            ts_output = Timeseries(output)
            self.Outputs.time_series.send(ts_output)
        else:
            self.infoa.setText(
                "No data on input yet, waiting to get something.")
            self.infob.setText('')
            self.Outputs.time_series.send(None)

    @staticmethod
    def smooth_zeros(arr: np.array) -> np.array:
        out = arr.copy()
        window = []
        zero_len = 0
        avg = 0
        stdev = 0
        for i in range(len(arr)):
            if avg > 0 and arr[i] == 0:
                zero_len += 1
            elif avg > 0 and arr[i] < avg - (8*stdev):
                zero_len += 1
            else:
                if zero_len > 0:
                    fillin = np.random.normal(avg, stdev, zero_len)
                    out[i-zero_len:i] = fillin
                    zero_len = 0
                window.append(arr[i])
                if len(window) > 40:
                    window.pop(0)
                avg = np.average(window)
                stdev = np.std(window)
        return out

if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview

    WidgetPreview(OWCleanWFDBSignal).run(
        Table("/home/chris/code/orange/wfdb_test.tab"))



from Orange.data import (ContinuousVariable, Domain, FileFormat, StringVariable, Table, TimeVariable)
import wfdb
import numpy as np
import os

class HDRReader_WFDB(FileFormat):
    """ Reader for WFDB files from PhysioNet.
    It is assumed that there are .dat files with binary data in the same
    directory as the .hea file. For 1001.hea, a file called 1001.dat.
    """
    EXTENSIONS = ('.hea',)
    DESCRIPTION = 'WFDB .hea+.dat files'
    
    def read(self):
        """ Fast reading of spectra. Return spectral information
        in two arrays (wavelengths and values). Only additional
        attributes (usually metas) are returned as a Table.

        Return a triplet:
            - 1D numpy array, (X)
            - 2D numpy array with the same last dimension as xs, (Y)
            - Orange.data.Table with only meta or class attributes
        """
        return HDRReader_WFDB.read_hea(self.filename)
    
    @staticmethod
    def read_hea(filename):
        rec = wfdb.io.rdrecord(filename.replace('.hea', ''))
        rec_name = os.path.basename(filename).replace('.hea', '')

        signal_names = rec.sig_name
        signal_data = rec.p_signal
        signal_length = rec.sig_len
        signal_units = rec.units
        samples_per_second = rec.fs

        timestamps = np.linspace(0, signal_length / samples_per_second, signal_length, endpoint=False)
                
        metas = [
            StringVariable("Record"),
            TimeVariable("Timestamp")
        ]
        for signal_name in signal_names:
            metas.append(ContinuousVariable(signal_name))
            metas.append(StringVariable(signal_name+" units"))
            
        domain = Domain([], None, metas=metas)
        
        metadata = []
        for i, row in enumerate(signal_data):
            rec = [rec_name, timestamps[i]]
            for j, name in enumerate(signal_names):
                rec.append(row[j])
                rec.append(signal_units[j])
            metadata.append(rec)
        
        meta_data = Table.from_numpy(
            domain,
            X=np.zeros((signal_length, 0)),
            metas=np.asarray(metadata, dtype=object)
        )
        
        return meta_data
    
    #read = read_spectra
    
if __name__ == "__main__":
    FileFormat.readers['.hea'] = HDRReader_WFDB
    t = Table("/home/chris/Downloads/ctu-chb-intrapartum-cardiotocography-database-1.0.0/1001.hea")
    print(t)
    